@@ -3,15 +3,14 @@
 # This file is part of the ruby-dbus project
 # Copyright (C) 2007 Arnaud Cornet and Paul van Tilburg
 #
-# This library is free software; you can redistribute it and/or
+# This library is free software; you caan redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
 # License, version 2.1 as published by the Free Software Foundation.
 # See the file "COPYING" for the exact licensing terms.
 
-require 'socket'
-require 'thread'
-require 'singleton'
-require 'fcntl'
+require "socket"
+require "thread"
+require "singleton"
 
 # = D-Bus main module
 #
@@ -29,7 +28,8 @@ module DBus
 
     # Create a new service with a given _name_ on a given _bus_.
     def initialize(name, bus)
-      @name, @bus = name, bus
+      @name = name
+      @bus = bus
       @root = Node.new("/")
     end
 
@@ -41,19 +41,28 @@ module DBus
     # Perform an introspection on all the objects on the service
     # (starting recursively from the root).
     def introspect
-      if block_given?
-        raise NotImplementedError
-      else
-        rec_introspect(@root, "/")
-      end
+      raise NotImplementedError if block_given?
+
+      rec_introspect(@root, "/")
       self
     end
 
-    # Retrieves an object (ProxyObject) at the given _path_.
-    def object(path)
-      node = get_node(path, true)
-      if node.object.nil?
-        node.object = ProxyObject.new(@bus, @name, path)
+    # Retrieves an object at the given _path_.
+    # @return [ProxyObject]
+    def [](path)
+      object(path, api: ApiOptions::A1)
+    end
+
+    # Retrieves an object at the given _path_
+    # whose methods always return an array.
+    # @return [ProxyObject]
+    def object(path, api: ApiOptions::A0)
+      node = get_node(path, _create = true)
+      if node.object.nil? || node.object.api != api
+        node.object = ProxyObject.new(
+          @bus, @name, path,
+          api: api
+        )
       end
       node.object
     end
@@ -68,40 +77,39 @@ module DBus
     # Raises ArgumentError if it is not a DBus::Object.
     # Returns the object, or false if _obj_ was not exported.
     def unexport(obj)
-      raise ArgumentError.new("DBus::Service#unexport() expects a DBus::Object argument") unless obj.kind_of?(DBus::Object)
+      raise ArgumentError, "DBus::Service#unexport() expects a DBus::Object argument" unless obj.is_a?(DBus::Object)
       return false unless obj.path
-      pathSep = obj.path.rindex("/") #last path seperator
-      parent_path = obj.path[1..pathSep-1]
-      node_name = obj.path[pathSep+1..-1]
+      last_path_separator_idx = obj.path.rindex("/")
+      parent_path = obj.path[1..last_path_separator_idx - 1]
+      node_name = obj.path[last_path_separator_idx + 1..-1]
 
       parent_node = get_node(parent_path, false)
       return false unless parent_node
       obj.service = nil
-      parent_node.delete(node_name)
+      parent_node.delete(node_name).object
     end
-	
+
     # Get the object node corresponding to the given _path_. if _create_ is
     # true, the the nodes in the path are created if they do not already exist.
     def get_node(path, create = false)
       n = @root
-      path.sub(/^\//, "").split("/").each do |elem|
-        if not n[elem]
-          if not create
-            return nil
-          else
-            n[elem] = Node.new(elem)
-          end
+      path.sub(%r{^/}, "").split("/").each do |elem|
+        if !(n[elem])
+          return nil if !create
+          n[elem] = Node.new(elem)
         end
         n = n[elem]
       end
       if n.nil?
-        puts "Warning, unknown object #{path}" if $DEBUG
+        DBus.logger.debug "Warning, unknown object #{path}"
       end
       n
     end
 
     #########
+
     private
+
     #########
 
     # Perform a recursive retrospection on the given current _node_
@@ -111,16 +119,15 @@ module DBus
       intfs, subnodes = IntrospectXMLParser.new(xml).parse
       subnodes.each do |nodename|
         subnode = node[nodename] = Node.new(nodename)
-        if path == "/"
-          subpath = "/" + nodename
-        else
-          subpath = path + "/" + nodename
-        end
+        subpath = if path == "/"
+                    "/" + nodename
+                  else
+                    path + "/" + nodename
+                  end
         rec_introspect(subnode, subpath)
       end
-      if intfs.size > 0
-        node.object = ProxyObjectFactory.new(xml, @bus, @name, path).build
-      end
+      return if intfs.empty?
+      node.object = ProxyObjectFactory.new(xml, @bus, @name, path).build
     end
   end
 
@@ -146,18 +153,18 @@ module DBus
 "http://www.freedesktop.org/standards/dbus/1.0/introspect.dtd">
 <node>
 '
-      self.each_pair do |k, v|
+      each_pair do |k, _v|
         xml += "<node name=\"#{k}\" />"
       end
       if @object
-        @object.intfs.each_pair do |k, v|
-          xml += %{<interface name="#{v.name}">\n}
+        @object.intfs.each_pair do |_k, v|
+          xml += %(<interface name="#{v.name}">\n)
           v.methods.each_value { |m| xml += m.to_xml }
           v.signals.each_value { |m| xml += m.to_xml }
-          xml +="</interface>\n"
+          xml += "</interface>\n"
         end
       end
-      xml += '</node>'
+      xml += "</node>"
       xml
     end
 
@@ -170,8 +177,8 @@ module DBus
     # Return instance inspect information, used by Node#inspect.
     def sub_inspect
       s = ""
-      if not @object.nil?
-        s += "%x " % @object.object_id
+      if !@object.nil?
+        s += format("%x ", @object.object_id)
       end
       s + "{" + keys.collect { |k| "#{k} => #{self[k].sub_inspect}" }.join(",") + "}"
     end
@@ -186,8 +193,8 @@ module DBus
   class Connection
     # The unique name (by specification) of the message.
     attr_reader :unique_name
-    # The socket that is used to connect with the bus.
-    attr_reader :socket
+    # pop and push messages here
+    attr_reader :message_queue
 
     # Create a new connection to the bus for a given connect _path_. _path_
     # format is described in the D-Bus specification:
@@ -196,99 +203,34 @@ module DBus
     # "transport1:key1=value1,key2=value2;transport2:key1=value1,key2=value2"
     # e.g. "unix:path=/tmp/dbus-test" or "tcp:host=localhost,port=2687"
     def initialize(path)
-      @path = path
+      @message_queue = MessageQueue.new(path)
       @unique_name = nil
-      @buffer = ""
-      @method_call_replies = Hash.new
-      @method_call_msgs = Hash.new
-      @signal_matchrules = Hash.new
+      @method_call_replies = {}
+      @method_call_msgs = {}
+      @signal_matchrules = {}
       @proxy = nil
       @object_root = Node.new("/")
-      @is_tcp = false
     end
 
-    # Connect to the bus and initialize the connection.
-    def connect
-      addresses = @path.split ";"
-      # connect to first one that succeeds
-      worked = addresses.find do |a|
-        transport, keyvaluestring = a.split ":"
-        kv_list = keyvaluestring.split ","
-        kv_hash = Hash.new
-        kv_list.each do |kv|
-          key, escaped_value = kv.split "="
-          value = escaped_value.gsub(/%(..)/) {|m| [$1].pack "H2" }
-          kv_hash[key] = value
-        end
-        case transport
-          when "unix"
-          connect_to_unix kv_hash
-          when "tcp"
-          connect_to_tcp kv_hash
-          else
-          # ignore, report?
-        end
+    # Dispatch all messages that are available in the queue,
+    # but do not block on the queue.
+    # Called by a main loop when something is available in the queue
+    def dispatch_message_queue
+      while (msg = @message_queue.pop(:non_block)) # FIXME: EOFError
+        process(msg)
       end
-      worked
-      # returns the address that worked or nil.
-      # how to report failure?
-    end
-
-    # Connect to a bus over tcp and initialize the connection.
-    def connect_to_tcp(params)
-      #check if the path is sufficient
-      if params.key?("host") and params.key?("port")
-        begin
-          #initialize the tcp socket
-          @socket = TCPSocket.new(params["host"],params["port"].to_i)
-          @socket.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
-          init_connection
-          @is_tcp = true
-        rescue
-          puts "Error: Could not establish connection to: #{@path}, will now exit."
-          exit(0) #a little harsh
-        end
-      else
-        #Danger, Will Robinson: the specified "path" is not usable
-        puts "Error: supplied path: #{@path}, unusable! sorry."
-      end
-    end
-
-    # Connect to an abstract unix bus and initialize the connection.
-    def connect_to_unix(params)
-      @socket = Socket.new(Socket::Constants::PF_UNIX,Socket::Constants::SOCK_STREAM, 0)
-      @socket.fcntl(Fcntl::F_SETFD, Fcntl::FD_CLOEXEC)
-      if ! params['abstract'].nil?
-        if HOST_END == LIL_END
-          sockaddr = "\1\0\0#{params['abstract']}"
-        else
-          sockaddr = "\0\1\0#{params['abstract']}"
-        end
-      elsif ! params['path'].nil?
-        sockaddr = Socket.pack_sockaddr_un(params['path'])
-      end
-      @socket.connect(sockaddr)
-      init_connection
-    end
-
-    # Send the buffer _buf_ to the bus using Connection#writel.
-    def send(buf)
-      @socket.write(buf) unless @socket.nil?
     end
 
     # Tell a bus to register itself on the glib main loop
     def glibize
-      require 'glib2'
+      require "glib2"
       # Circumvent a ruby-glib bug
-      @channels ||= Array.new
+      @channels ||= []
 
-      gio = GLib::IOChannel.new(@socket.fileno)
+      gio = GLib::IOChannel.new(@message_queue.socket.fileno)
       @channels << gio
-      gio.add_watch(GLib::IOChannel::IN) do |c, ch|
-        update_buffer
-        messages.each do |msg|
-          process(msg)
-        end
+      gio.add_watch(GLib::IOChannel::IN) do |_c, _ch|
+        dispatch_message_queue
         true
       end
     end
@@ -381,9 +323,10 @@ module DBus
     </signal>
   </interface>
 </node>
-'
-# This apostroph is for syntax highlighting editors confused by above xml: "
+'.freeze
+    # This apostroph is for syntax highlighting editors confused by above xml: "
 
+    # @api private
     # Send a _message_.
     # If _reply_handler_ is not given, wait for the reply
     # and return the reply, or raise the error.
@@ -394,11 +337,8 @@ module DBus
       ret = nil
       if reply_handler.nil?
         send_sync(message) do |rmsg|
-          if rmsg.is_a?(Error)
-            raise rmsg
-          else
-            ret = rmsg.params
-          end
+          raise rmsg if rmsg.is_a?(Error)
+          ret = rmsg.params
         end
       else
         on_return(message) do |rmsg|
@@ -408,11 +348,12 @@ module DBus
             reply_handler.call(rmsg, * rmsg.params)
           end
         end
-        send(message.marshall)
+        @message_queue.push(message)
       end
       ret
     end
 
+    # @api private
     def introspect_data(dest, path, &reply_handler)
       m = DBus::Message.new(DBus::Message::METHOD_CALL)
       m.path = path
@@ -424,14 +365,15 @@ module DBus
         send_sync_or_async(m).first
       else
         send_sync_or_async(m) do |*args|
-          # TODO test async introspection, is it used at all?
-          args.shift            # forget the message, pass only the text
+          # TODO: test async introspection, is it used at all?
+          args.shift # forget the message, pass only the text
           reply_handler.call(*args)
           nil
         end
       end
     end
 
+    # @api private
     # Issues a call to the org.freedesktop.DBus.Introspectable.Introspect method
     # _dest_ is the service and _path_ the object path you want to introspect
     # If a code block is given, the introspect call in asynchronous. If not
@@ -441,7 +383,7 @@ module DBus
     # The returned object is a ProxyObject that has methods you can call to
     # issue somme METHOD_CALL messages, and to setup to receive METHOD_RETURN
     def introspect(dest, path)
-      if not block_given?
+      if !block_given?
         # introspect in synchronous !
         data = introspect_data(dest, path)
         pof = DBus::ProxyObjectFactory.new(data, self, dest, path)
@@ -461,120 +403,64 @@ module DBus
     #
     # FIXME, NameRequestError cannot really be rescued as it will be raised
     # when dispatching a later call. Rework the API to better match the spec.
+    # @return [Service]
     def request_service(name)
       # Use RequestName, but asynchronously!
       # A synchronous call would not work with service activation, where
       # method calls to be serviced arrive before the reply for RequestName
       # (Ticket#29).
       proxy.RequestName(name, NAME_FLAG_REPLACE_EXISTING) do |rmsg, r|
-        if rmsg.is_a?(Error)  # check and report errors first
-	  raise rmsg
-	elsif r != REQUEST_NAME_REPLY_PRIMARY_OWNER
-          raise NameRequestError
-	end
+        # check and report errors first
+        raise rmsg if rmsg.is_a?(Error)
+        raise NameRequestError unless r == REQUEST_NAME_REPLY_PRIMARY_OWNER
       end
       @service = Service.new(name, self)
       @service
     end
 
     # Set up a ProxyObject for the bus itself, since the bus is introspectable.
+    # @return [ProxyObject] that always returns an array
+    #   ({DBus::ApiOptions#proxy_method_returns_array})
     # Returns the object.
     def proxy
-      if @proxy == nil
+      if @proxy.nil?
         path = "/org/freedesktop/DBus"
         dest = "org.freedesktop.DBus"
-        pof = DBus::ProxyObjectFactory.new(DBUSXMLINTRO, self, dest, path)
+        pof = DBus::ProxyObjectFactory.new(
+          DBUSXMLINTRO, self, dest, path,
+          api: ApiOptions::A0
+        )
         @proxy = pof.build["org.freedesktop.DBus"]
       end
       @proxy
     end
 
-    # Fill (append) the buffer from data that might be available on the
-    # socket.
-    def update_buffer
-      @buffer += @socket.read_nonblock(MSG_BUF_SIZE)  
-    rescue EOFError
-      raise                     # the caller expects it
-    rescue Exception => e
-      puts "Oops:", e
-      raise if @is_tcp          # why?
-      puts "WARNING: read_nonblock failed, falling back to .recv"
-      @buffer += @socket.recv(MSG_BUF_SIZE)  
-    end
-
-    # Get one message from the bus and remove it from the buffer.
-    # Return the message.
-    def pop_message
-      return nil if @buffer.empty?
-      ret = nil
-      begin
-        ret, size = Message.new.unmarshall_buffer(@buffer)
-        @buffer.slice!(0, size)
-      rescue IncompleteBufferException => e
-        # fall through, let ret be null
-      end
-      ret
-    end
-
-    # Retrieve all the messages that are currently in the buffer.
-    def messages
-      ret = Array.new
-      while msg = pop_message
-        ret << msg
-      end
-      ret
-    end
-
-    # The buffer size for messages.
-    MSG_BUF_SIZE = 4096
-
-    # Update the buffer and retrieve all messages using Connection#messages.
-    # Return the messages.
-    def poll_messages
-      ret = nil
-      r, d, d = IO.select([@socket], nil, nil, 0)
-      if r and r.size > 0
-        update_buffer
-      end
-      messages
-    end
-
+    # @api private
     # Wait for a message to arrive. Return it once it is available.
     def wait_for_message
-      if @socket.nil?
-        puts "ERROR: Can't wait for messages, @socket is nil."
-        return
-      end
-      ret = pop_message
-      while ret == nil
-        r, d, d = IO.select([@socket])
-        if r and r[0] == @socket
-          update_buffer
-          ret = pop_message
-        end
-      end
-      ret
+      @message_queue.pop # FIXME: EOFError
     end
 
+    # @api private
     # Send a message _m_ on to the bus. This is done synchronously, thus
     # the call will block until a reply message arrives.
     def send_sync(m, &retc) # :yields: reply/return message
-      return if m.nil? #check if somethings wrong
-      send(m.marshall)
+      return if m.nil? # check if somethings wrong
+      @message_queue.push(m)
       @method_call_msgs[m.serial] = m
       @method_call_replies[m.serial] = retc
 
       retm = wait_for_message
-      
-      return if retm.nil? #check if somethings wrong
-      
+      return if retm.nil? # check if somethings wrong
+
       process(retm)
-      while @method_call_replies.has_key? m.serial
+      while @method_call_replies.key? m.serial
         retm = wait_for_message
         process(retm)
       end
     end
 
+    # @api private
     # Specify a code block that has to be executed when a reply for
     # message _m_ is received.
     def on_return(m, &retc)
@@ -591,10 +477,10 @@ module DBus
     def add_match(mr, &slot)
       # check this is a signal.
       mrs = mr.to_s
-      puts "#{@signal_matchrules.size} rules, adding #{mrs.inspect}" if $DEBUG
+      DBus.logger.debug "#{@signal_matchrules.size} rules, adding #{mrs.inspect}"
       # don't ask for the same match if we override it
       unless @signal_matchrules.key?(mrs)
-        puts "Asked for a new match" if $DEBUG
+        DBus.logger.debug "Asked for a new match"
         proxy.AddMatch(mrs)
       end
       @signal_matchrules[mrs] = slot
@@ -602,23 +488,24 @@ module DBus
 
     def remove_match(mr)
       mrs = mr.to_s
-      unless @signal_matchrules.delete(mrs).nil?
-        # don't remove nonexisting matches.
-        # FIXME if we do try, the Error.MatchRuleNotFound is *not* raised
-        # and instead is reported as "no return code for nil"
-        proxy.RemoveMatch(mrs)
-      end
+      rule_existed = @signal_matchrules.delete(mrs).nil?
+      # don't remove nonexisting matches.
+      return if rule_existed
+      # FIXME: if we do try, the Error.MatchRuleNotFound is *not* raised
+      # and instead is reported as "no return code for nil"
+      proxy.RemoveMatch(mrs)
     end
 
+    # @api private
     # Process a message _m_ based on its type.
     def process(m)
-      return if m.nil? #check if somethings wrong
+      return if m.nil? # check if somethings wrong
       case m.message_type
       when Message::ERROR, Message::METHOD_RETURN
-        raise InvalidPacketException if m.reply_serial == nil
+        raise InvalidPacketException if m.reply_serial.nil?
         mcs = @method_call_replies[m.reply_serial]
-        if not mcs
-          puts "DEBUG: no return code for mcs: #{mcs.inspect} m: #{m.inspect}" if $DEBUG
+        if !mcs
+          DBus.logger.debug "no return code for mcs: #{mcs.inspect} m: #{m.inspect}"
         else
           if m.message_type == Message::ERROR
             mcs.call(Error.new(m))
@@ -630,23 +517,23 @@ module DBus
         end
       when DBus::Message::METHOD_CALL
         if m.path == "/org/freedesktop/DBus"
-          puts "DEBUG: Got method call on /org/freedesktop/DBus" if $DEBUG
+          DBus.logger.debug "Got method call on /org/freedesktop/DBus"
         end
         node = @service.get_node(m.path)
-        if not node
+        if !node
           reply = Message.error(m, "org.freedesktop.DBus.Error.UnknownObject",
                                 "Object #{m.path} doesn't exist")
-          send(reply.marshall)
+          @message_queue.push(reply)
         # handle introspectable as an exception:
-        elsif m.interface == "org.freedesktop.DBus.Introspectable" and
-            m.member == "Introspect"
+        elsif m.interface == "org.freedesktop.DBus.Introspectable" &&
+              m.member == "Introspect"
           reply = Message.new(Message::METHOD_RETURN).reply_to(m)
           reply.sender = @unique_name
           reply.add_param(Type::STRING, node.to_xml)
-          send(reply.marshall)
+          @message_queue.push(reply)
         else
           obj = node.object
-          return if obj.nil?    # FIXME, sends no reply
+          return if obj.nil? # FIXME, pushes no reply
           obj.dispatch(m) if obj
         end
       when DBus::Message::SIGNAL
@@ -657,18 +544,22 @@ module DBus
           end
         end
       else
-        puts "DEBUG: Unknown message type: #{m.message_type}" if $DEBUG
+        DBus.logger.debug "Unknown message type: #{m.message_type}"
       end
+    rescue Exception => ex
+      raise m.annotate_exception(ex)
     end
 
     # Retrieves the Service with the given _name_.
+    # @return [Service]
     def service(name)
       # The service might not exist at this time so we cannot really check
       # anything
       Service.new(name, self)
     end
-    alias :[] :service
+    alias [] service
 
+    # @api private
     # Emit a signal event for the given _service_, object _obj_, interface
     # _intf_ and signal _sig_ with arguments _args_.
     def emit(service, obj, intf, sig, *args)
@@ -682,7 +573,7 @@ module DBus
         m.add_param(par.type, args[i])
         i += 1
       end
-      send(m.marshall)
+      @message_queue.push(m)
     end
 
     ###########################################################################
@@ -697,19 +588,12 @@ module DBus
       m.member = "Hello"
       send_sync(m) do |rmsg|
         @unique_name = rmsg.destination
-        puts "Got hello reply. Our unique_name is #{@unique_name}" if $DEBUG
+        DBus.logger.debug "Got hello reply. Our unique_name is #{@unique_name}"
       end
       @service = Service.new(@unique_name, self)
     end
-
-    # Initialize the connection to the bus.
-    def init_connection
-      @client = Client.new(@socket)
-      @client.authenticate
-    end
   end # class Connection
 
-
   # = D-Bus session bus class
   #
   # The session bus is a session specific bus (mostly for desktop use).
@@ -719,19 +603,32 @@ module DBus
   class ASessionBus < Connection
     # Get the the default session bus.
     def initialize
-      super(ENV["DBUS_SESSION_BUS_ADDRESS"] || address_from_file)
-      connect
+      super(self.class.session_bus_address)
       send_hello
     end
 
-    def address_from_file
-      f = File.new("/var/lib/dbus/machine-id")
-      machine_id = f.readline.chomp
-      f.close
-      display = ENV["DISPLAY"].gsub(/.*:([0-9]*).*/, '\1')
-      File.open(ENV["HOME"] + "/.dbus/session-bus/#{machine_id}-#{display}").each do |line|
+    def self.session_bus_address
+      ENV["DBUS_SESSION_BUS_ADDRESS"] ||
+        address_from_file ||
+        "launchd:env=DBUS_LAUNCHD_SESSION_BUS_SOCKET"
+    end
+
+    def self.address_from_file
+      # systemd uses /etc/machine-id
+      # traditional dbus uses /var/lib/dbus/machine-id
+      machine_id_path = Dir["{/etc,/var/lib/dbus,/var/db/dbus}/machine-id"].first
+      return nil unless machine_id_path
+      machine_id = File.read(machine_id_path).chomp
+
+      display = ENV["DISPLAY"][/:(\d+)\.?/, 1]
+
+      bus_file_path = File.join(ENV["HOME"], "/.dbus/session-bus/#{machine_id}-#{display}")
+      return nil unless File.exist?(bus_file_path)
+
+      File.open(bus_file_path).each_line do |line|
         if line =~ /^DBUS_SESSION_BUS_ADDRESS=(.*)/
-          return $1
+          address = Regexp.last_match(1)
+          return address[/\A'(.*)'\z/, 1] || address[/\A"(.*)"\z/, 1] || address
         end
       end
     end
@@ -742,7 +639,6 @@ module DBus
     include Singleton
   end
 
-
   # = D-Bus system bus class
   #
   # The system bus is a system-wide bus mostly used for global or
@@ -754,28 +650,25 @@ module DBus
     # Get the default system bus.
     def initialize
       super(SystemSocketName)
-      connect
       send_hello
     end
   end
-  
+
   # = D-Bus remote (TCP) bus class
   #
-  # This class may be used when connecting to remote (listening on a TCP socket) 
+  # This class may be used when connecting to remote (listening on a TCP socket)
   # busses. You can also use it to connect to other non-standard path busses.
-  # 
+  #
   # The specified socket_name should look like this:
   # (for TCP)         tcp:host=127.0.0.1,port=2687
   # (for Unix-socket) unix:path=/tmp/my_funky_bus_socket
-  # 
-  # you'll need to take care about authentification then, more info here: 
+  #
+  # you'll need to take care about authentification then, more info here:
   # http://github.com/pangdudu/ruby-dbus/blob/master/README.rdoc
   class RemoteBus < Connection
-
     # Get the remote bus.
-    def initialize socket_name
+    def initialize(socket_name)
       super(socket_name)
-      connect
       send_hello
     end
   end
@@ -785,13 +678,15 @@ module DBus
     include Singleton
   end
 
-  # Shortcut for the SystemBus instance
-  def DBus.system_bus
+  # Shortcut for the {SystemBus} instance
+  # @return [Connection]
+  def self.system_bus
     SystemBus.instance
   end
 
-  # Shortcut for the SessionBus instance
-  def DBus.session_bus
+  # Shortcut for the {SessionBus} instance
+  # @return [Connection]
+  def self.session_bus
     SessionBus.instance
   end
 
@@ -803,13 +698,13 @@ module DBus
   class Main
     # Create a new main event loop.
     def initialize
-      @buses = Hash.new
+      @buses = {}
       @quitting = false
     end
 
     # Add a _bus_ to the list of buses to watch for events.
     def <<(bus)
-      @buses[bus.socket] = bus
+      @buses[bus.message_queue.socket] = bus
     end
 
     # Quit a running main loop, to be used eg. from a signal handler
@@ -822,22 +717,22 @@ module DBus
       # before blocking, empty the buffers
       # https://bugzilla.novell.com/show_bug.cgi?id=537401
       @buses.each_value do |b|
-        while m = b.pop_message
+        while (m = b.message_queue.message_from_buffer_nonblock)
           b.process(m)
         end
       end
-      while not @quitting and not @buses.empty?
+      while !@quitting && !@buses.empty?
         ready = IO.select(@buses.keys, [], [], 5) # timeout 5 seconds
         next unless ready # timeout exceeds so continue unless quitting
         ready.first.each do |socket|
           b = @buses[socket]
           begin
-            b.update_buffer
+            b.message_queue.buffer_from_socket_nonblock
           rescue EOFError, SystemCallError
             @buses.delete socket # this bus died
             next
           end
-          while m = b.pop_message
+          while (m = b.message_queue.message_from_buffer_nonblock)
             b.process(m)
           end
         end
